<html>
  <head>
    <title>pycobertura report</title>
    <meta charset="UTF-8">
    <style>
{% include 'normalize.css' %}
{% include 'skeleton.css' %}
.hit {background-color: #EAFFEA}
.miss {background-color: #FFECEC}
.container .code {margin-left: 0}
pre {line-height: 1.3}
    </style>
  </head>
  <body>
    <div class="container">
      <table class="u-full-width">
        <thead>
          <tr>
            <th>Name</th>
            <th>Stmts</th>
            <th>Miss</th>
            <th>Cover</th>
            <th>Missing</th>
          </tr>
        </thead>
        <tbody>
{%- for class_name, total_lines, total_misses, line_rate, missing_lines in lines %}
          <tr>
            <td><a href="#{{ class_name }}">{{ class_name }}</a></td>
            <td>{{ total_lines }}</td>
            <td>{{ total_misses }}</td>
            <td>{{ line_rate }}</td>
            <td>{{ missing_lines }}</td>
          </tr>
{%- endfor %}
        </tbody>
        <tfoot>
          <tr>
            <td>{{ footer[0] }}</td>
            <td>{{ footer[1] }}</td>
            <td>{{ footer[2] }}</td>
            <td>{{ footer[3] }}</td>
            <td>{{ footer[4] }}</td>
          </tr>
        </tfoot>
      </table>
{%- from 'macro.source.jinja2' import render_source -%}
{%- for class_name, filename, source in sources %}
{{ render_source(class_name, source) }}
{%- endfor %}
    </div>
  </body>
</html>
